#include "bezierwindow.h"

// Message handler for about box.
INT_PTR CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{	
	switch (message)
	{
	case WM_INITDIALOG:
		return (INT_PTR)TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
		{
			EndDialog(hDlg, LOWORD(wParam));
			return (INT_PTR)TRUE;
		}
		break;
	}
	return (INT_PTR)FALSE;
}

// Message handler for the Instructions dialog
INT_PTR CALLBACK Instructions(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	char* InstructionText = 

		// Instructions go here:


	"Camera & Movement:\
	Right click on the mouse to activate camera. The mouse will control the direction of view. W/S will move back and forwards respectively, A/D will move left and right respectively and Q/E will rise up and down respectively.\n\n"
	"Light Source:\
	The 3 yellow balls are the light source. You are free to click and drag them anywhere in the world.\n\n"
	"Control Points:\
	Click and drag the move the control points about. You can activate the camera to move around so that you can freely position the control points. The control points can also be reduced via the menu.\n\n"
	"Coordinates:\
	Select a Control Point and press the Enter key to manually edit the world coordinates \n\n"
	"Menu:\
	Most of the controls are easily accessible through the menu bar.\n\n"
	"Press F1 to reset scene.\n\n";

	//

	switch (message)
	{
	case WM_INITDIALOG:
		SetDlgItemText(hDlg, IDC_INSTRUCTIONS, InstructionText);
		return (INT_PTR)TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
		{
			EndDialog(hDlg, LOWORD(wParam));
			return (INT_PTR)TRUE;
		}
		break;
	}
	return (INT_PTR)FALSE;
}

// Message handler for about box. Edit coordinates...
INT_PTR CALLBACK EditCoordinate(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		{
			Vector3* vec = (Vector3*)lParam;

			cout << vec->m_X << " ";
			cout << vec->m_Y << " ";
			cout << vec->m_Z << endl;
			
			SetDlgItemText(hDlg, IDC_X, ToString(vec->m_X).c_str());
			SetDlgItemText(hDlg, IDC_Y, ToString(vec->m_Y).c_str());
			SetDlgItemText(hDlg, IDC_Z, ToString(vec->m_Z).c_str());

			return (INT_PTR)TRUE;
		}

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK)
		{
			Vector3* v = new Vector3();

			char x[11] = {0};
			char y[11] = {0};
			char z[11] = {0};
			
			GetDlgItemText(hDlg, IDC_X, x, sizeof(x)+1);
			GetDlgItemText(hDlg, IDC_Y, y, sizeof(y)+1);
			GetDlgItemText(hDlg, IDC_Z, z, sizeof(z)+1);

			v->m_X = ParseString<float>(x);
			v->m_Y = ParseString<float>(y);
			v->m_Z = ParseString<float>(z);

			EndDialog(hDlg, (INT_PTR)v);
		}

		if (LOWORD(wParam) == IDCANCEL) EndDialog(hDlg, (INT_PTR)NULL);

		break;
	}
	return (INT_PTR)FALSE;
}

//

BezierWindow::BezierWindow() : selected(0)
{
	HICON	MainIcon	= LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_BEZIER));
	HICON	SmallIcon	= LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_SMALL));

	create(this, "Bezier Curve", MAKEINTRESOURCE(IDC_BEZIER), MainIcon, SmallIcon, false, NULL);

	InitOpenGL();

	Create();

	//

	show(true);
}

void BezierWindow::Destroy()
{
	World::Destroy();
}

void BezierWindow::Create()
{
	// Default menu options...

	CheckMenuItem(menu(), ID_SCENE_SKYBOX, MF_CHECKED);

	CheckMenuItem(menu(), ID_SPLINE_HIDELINES, MF_UNCHECKED);

	CheckMenuItem(menu(), ID_CALULATINGMETHOD_PARAMETRICAL, MF_CHECKED);
	CheckMenuItem(menu(), ID_CALULATINGMETHOD_CASTLEJAU, MF_UNCHECKED);

	CheckMenuItem(menu(), ID_CAMERA_1STPERSON, MF_UNCHECKED);
	CheckMenuItem(menu(), ID_CAMERA_3RDPERSON, MF_CHECKED);

	CheckMenuItem(menu(), ID_OBJECT_SHOW, MF_CHECKED);

	CheckMenuItem(menu(), ID_PHYSICS_ON, MF_CHECKED);

	CheckMenuItem(menu(), ID_ANIMATION_STOP, MF_UNCHECKED);
	CheckMenuItem(menu(), ID_ANIMATION_LOOP, MF_UNCHECKED);

	// Objects...

	ResetCamera();

	World::Create(&camera);
}

BezierWindow::~BezierWindow()
{
	Destroy();
}

void BezierWindow::ResetCamera()
{
	camera = Camera();

	camera.Position().m_Y -= 100.0f;
	camera.Position().m_Z += 30.0f;
}

//

void BezierWindow::Render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear screen and depth buffer.

	glLoadIdentity();

	//

	Step();	// Physics

	// Get the fighter to align the camera if it has been asked to.

	fighter.AlignToCamera();

	camera.render();
	World::Render();
}

Vector3 BezierWindow::cursorTo3D(int x, int y, Vector3 p)
{
	int		viewport[4];
	double	modelview[16];
	double	projection[16];
	float	winX, winY;
	double	posX, posY, posZ;

	glPushMatrix();	// We are going to render the camera here
					// so we need restore the model view matrix once done.

	glLoadIdentity();

	camera.render();

	glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
	glGetDoublev(GL_PROJECTION_MATRIX, projection);
	glGetIntegerv(GL_VIEWPORT, viewport);

	winX = (float)x;
	winY = height() - (float)y;

	double rayX;
	double rayY;
	double rayZ;

	gluProject(p.m_X, p.m_Y, p.m_Z, modelview, projection, viewport, &rayX, &rayY, &rayZ);

	//glReadPixels(x, int(winY), 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &winZ);
	gluUnProject(winX, winY, rayZ, modelview, projection, viewport, &posX, &posY, &posZ);

	glPopMatrix();

	return Vector3((float)posX, (float)posY, (float)posZ);
}

//

void BezierWindow::CloseEvent() // We want the application to end when we are closed.
{		
	PostQuitMessage(0);
}

void BezierWindow::MouseDownEvent(DWORD key, int x, int y)
{
	if(key & MK_RBUTTON) camera.setCapture(!camera.captured());

	if(!camera.captured() && key & MK_LBUTTON)	// The user is trying to select a control point or light.
	{
		// GL assumes bottom left coordinates.
		GLObject* obj = PickObject(camera, x, height()-y);

		// De-select previous only if its a control point.
		if(IsControlPoint(selected))
		{
			cout << "previous is control point" << endl;

			((ControlPoint*)selected)->deSelect();
		}

		if(IsControlPoint(obj) || IsLight(obj)) selected = obj;
		else selected = 0;

		if(selected)
		{
			cout << "Picked point <" << selected->m_X << ", " << selected->m_Y << ", " << selected->m_Z << ">" << endl;
			if(IsControlPoint(selected)) ((ControlPoint*)selected)->select();
		}
	}
}

void BezierWindow::KeyDownEvent(BYTE key)
{
	if(key == VK_ESCAPE) camera.release();

	if(key == VK_F1) Create();

	// Only interested if we are not moving around and the selected point is a control point.
	if(key == VK_RETURN && !camera.captured() && IsControlPoint(selected))
	{
		Vector3* vec = (Vector3*)DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_EDITCOORDINATE), handle(), EditCoordinate, (LPARAM)((Vector3*)selected));

		if(!vec) return;	// We canceled the edit.
		
		(Vector3&)(*selected) = *vec;

		spline.build();	// Rebuild in case we edited it.

		delete vec;	// Delete the returned vector.
	}
	if (key == VK_SPACE)
	{
		spline.Time();
	
	}

}

void BezierWindow::MouseMoveEvent(DWORD key, int x, int y)
{
	if(key & MK_LBUTTON && selected)
	{			
		if(IsControlPoint(selected))
		{
			// We have to include the spline offset in the pick transformation.
			
			(Vector3&)(*selected) = cursorTo3D(x, y, *(Vector3*)selected + (Vector3)spline);
			(Vector3&)(*selected) -= (Vector3)spline;

			spline.build();	// Dont bother if we are just moving a light.
		}
		else // Its a light.
		{
			(Vector3&)(*selected) = cursorTo3D(x, y, *(Vector3*)selected);
		}
	}
}

//

BOOL BezierWindow::handleMessage(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam, LRESULT *returnCode)
{
	switch(message)
	{
	case WM_COMMAND:

		switch (LOWORD(wParam))
		{
		case ID_HELP_ABOUT:
			DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_ABOUT), hWnd, About);
			break;
		case ID_HELP_INSTRUCTIONS:
			DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_INSTRUCTIONS), hWnd, Instructions);
			break;
		case ID_FILE_EXIT:
			destroy();
			CloseEvent();	// We want to post a quit message so the mesage loop ends.
			break;

		case ID_SCENE_RESET:
			
			cout << "Resetting scene..." << endl;
			
			Create();	// Reset everything!

			cout << "Scene reset!" << endl;

			break;

		case ID_SCENE_SKYBOX:

			skybox.Visible() = (GetMenuState(menu(), ID_SCENE_SKYBOX, MF_BYCOMMAND) == MF_CHECKED) ? false : true;
			CheckMenuItem(menu(), ID_SCENE_SKYBOX, skybox.Visible() ? MF_CHECKED : MF_UNCHECKED);

			break;

		case ID_SPLINE_ADDPOINT:

			{
				GLObject* point = new ControlPoint(Vector3(5.0f, 0.0f, 0.0f));

				if(!spline.empty()) *(Vector3*)point += (*(Vector3*)spline[spline.size()-1]);

				spline.push_back(point);

				spline.build();	// Update all the points etc.

				// Parametric evaluation becomes numerically
				// unstable if we have more than 12 points, so switch to a more reliable method.
				// Just to be safe we set the limit at 10...
				if(!spline.Casteljau() && spline.size() > 10)
				{
					say("Too many points for Parametric method (numerical instability). Max allowed is 10. Switching to Casteljau algorithm...");
					
					spline.Casteljau() = true;

					CheckMenuItem(menu(), ID_CALULATINGMETHOD_PARAMETRICAL, spline.Casteljau() ? MF_UNCHECKED : MF_CHECKED);
					CheckMenuItem(menu(), ID_CALULATINGMETHOD_CASTLEJAU, spline.Casteljau() ? MF_CHECKED : MF_UNCHECKED);
				}
			}
			break;

		case ID_SPLINE_REMOVEPOINT:

			if(!spline.empty()) spline.erase(spline.end()-1);
			if(!IsControlPoint(selected) || !IsLight(selected)) selected = NULL;	// We removed the selected point.
			break;

		case ID_SPLINE_HIDELINES:

			spline.ShowLines() = (GetMenuState(menu(), ID_SPLINE_HIDELINES, MF_BYCOMMAND) == MF_UNCHECKED) ? true : false;
			spline.ShowLines() = !spline.ShowLines();
			CheckMenuItem(menu(), ID_SPLINE_HIDELINES, spline.ShowLines() ? MF_UNCHECKED : MF_CHECKED);

		case ID_SPLINE_INCREASESMOOTH:
			spline.dt() *= 0.5;
			break;

		case ID_SPLINE_DECREASESMOOTH:
			spline.dt() *= 2.0f;
			break;

		case ID_CALULATINGMETHOD_PARAMETRICAL:

			// Only allow if there are fewer than a certain number of points.
			if(spline.size() <= 10) spline.Casteljau() = false;
			else
			{
				spline.Casteljau() = true;
				say("Too many points for Parametric method (numerical instability). Max allowd is 10.", TRUE);
			}

			CheckMenuItem(menu(), ID_CALULATINGMETHOD_PARAMETRICAL, spline.Casteljau() ? MF_UNCHECKED : MF_CHECKED);
			CheckMenuItem(menu(), ID_CALULATINGMETHOD_CASTLEJAU, spline.Casteljau() ? MF_CHECKED : MF_UNCHECKED);

			break;

		case ID_CALULATINGMETHOD_CASTLEJAU:

			spline.Casteljau() = true;

			CheckMenuItem(menu(), ID_CALULATINGMETHOD_PARAMETRICAL, spline.Casteljau() ? MF_UNCHECKED : MF_CHECKED);
			CheckMenuItem(menu(), ID_CALULATINGMETHOD_CASTLEJAU, spline.Casteljau() ? MF_CHECKED : MF_UNCHECKED);

			break;

		case ID_CAMERA_1STPERSON:
			camera.UpdateVectors() = false;	// The fighter will be calculating the forward vector.
			fighter.Track(&camera);			// Ask the fighter to track the camera.

			CheckMenuItem(menu(), ID_CAMERA_1STPERSON, MF_CHECKED);
			CheckMenuItem(menu(), ID_CAMERA_3RDPERSON, MF_UNCHECKED);

			break;

		case ID_CAMERA_3RDPERSON:
			camera.UpdateVectors() = true;		// The camera must update and calculate its own forward and vector position.
			fighter.Track(NULL);				// We dont want the fighter to track.

			CheckMenuItem(menu(), ID_CAMERA_1STPERSON, MF_UNCHECKED);
			CheckMenuItem(menu(), ID_CAMERA_3RDPERSON, MF_CHECKED);

			ResetCamera();

			break;

		case ID_OBJECT_SHOW:
			fighter.Visible() = (GetMenuState(menu(), ID_OBJECT_SHOW, MF_BYCOMMAND) == MF_CHECKED) ? true : false;
			fighter.Visible() = !fighter.Visible();
			CheckMenuItem(menu(), ID_OBJECT_SHOW, fighter.Visible() ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_PHYSICS_ON:
			PhysicsEnabled() = (GetMenuState(menu(), ID_PHYSICS_ON, MF_BYCOMMAND) == MF_CHECKED) ? true : false;
			PhysicsEnabled() = !PhysicsEnabled();
			CheckMenuItem(menu(), ID_PHYSICS_ON, PhysicsEnabled() ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_PHYSICS_INCREASEGRAVITY:
			g *= 1.1f;	// 10% increase.
			break;
		case ID_PHYSICS_DECREASEGRAVITY:
			g *= 0.9f;	// 10% decrease.
			break;

		case ID_ANIMATION_START:
			w = 0.0f;	// Reset at the begining of the path.
			u = 5.0f;	// Give it some initial velocity.
			break;

		case ID_ANIMATION_STOP:
			Paused() = (GetMenuState(menu(), ID_ANIMATION_STOP, MF_BYCOMMAND) == MF_CHECKED) ? true : false;
			Paused() = !Paused();
			CheckMenuItem(menu(), ID_ANIMATION_STOP, Paused() ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_ANIMATION_LOOP:
			Looped() = (GetMenuState(menu(), ID_ANIMATION_LOOP, MF_BYCOMMAND) == MF_CHECKED) ? true : false;
			Looped() = !Looped();
			CheckMenuItem(menu(), ID_ANIMATION_LOOP, Looped() ? MF_CHECKED : MF_UNCHECKED);
			break;

		case ID_LIGHTS_INCREASEINTENSITY:
			light1.Linear() *= 0.9f;		// Its a falloff so to increase light, we decrease this.
			light2.Linear() *= 0.9f;
			light3.Linear() *= 0.9f;
			break;

		case ID_LIGHTS_DECREASEINTENSITY:
			light1.Linear() *= 1.1f;		// Its a falloff so to decrease light, we increase this.
			light2.Linear() *= 1.1f;
			light3.Linear() *= 1.1f;
			break;

		default:
			return FALSE;	// We didnt handle this command.
		}

		break;

		default:

			return FALSE;	// We didnt handle this message.
	}

	return TRUE; // We handled the message.
}