#include "globject.h"

GLObject::GLObject() : Vector3(), m_Visible(true)
{
	setScale(1.0f);
	setDiffuse(1.0f, 1.0f, 1.0f);
	setSpecular(0.0f, 0.0f, 0.0f);
	setShininess(50);
}

GLObject::GLObject(Vector3 v) : Vector3(v), m_Visible(true)
{
	setScale(1.0f);
}

void GLObject::Destroy()	// Release all data this object holds.
{
}

void GLObject::light()
{		
	glColor3f(diffuse[0], diffuse[1], diffuse[2]);
	
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, &shininess);
}

void GLObject::transform()
{
	glTranslatef(m_X, m_Y, m_Z);
	glScalef(sx, sy, sz);
}

void GLObject::save()		// Save state information before render.
{
	glPushAttrib(GL_ENABLE_BIT);
	glPushAttrib(GL_TEXTURE_BIT);
	glPushAttrib(GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
}

void GLObject::restore()	// Restore state information.
{
	glPopMatrix();

	glPopAttrib();
	glPopAttrib();
	glPopAttrib();
}

void GLObject::Render()
{
	glLoadName((GLuint)this);
}

void GLObject::setShininess(float s)
{
	shininess = s;
}

void GLObject::setDiffuse(float r, float g, float b, float a)
{
	diffuse[0] = r;
	diffuse[1] = g;
	diffuse[2] = b;
	diffuse[3] = a;
}

void GLObject::setSpecular(float r, float g, float b, float a)
{
	specular[0] = r;
	specular[1] = g;
	specular[2] = b;
	specular[3] = a;
}

void GLObject::setScale(float x, float y, float z)
{
	this->sx = x;
	this->sy = y;
	this->sz = z;
}

void GLObject::setScale(float s)
{
	setScale(s, s, s);
}

bool& GLObject::Visible()
{
	return m_Visible;
}

//

bool GLObject::GLLoadBitmap(const char* filename, GLuint& texid) // Creates texture from a bitmap file.
{
	HBITMAP hBitmap;
	BITMAP	Bitmap;

	glGenTextures(1, &texid); // Create the texture
	hBitmap = (HBITMAP)LoadImage(GetModuleHandle(NULL), filename,
		IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION | LR_LOADFROMFILE);

	if (!hBitmap)	return FALSE;

	GetObject(hBitmap, sizeof(Bitmap), &Bitmap);

	glPixelStorei(GL_UNPACK_ALIGNMENT, 4);	// 4 Byte alignment

	glBindTexture(GL_TEXTURE_2D, texid);								// Bind to id.
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);	// Linear min filter
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);	// Linear mag filter
	glTexImage2D(GL_TEXTURE_2D, 0, 3, Bitmap.bmWidth, Bitmap.bmHeight,
		0, GL_BGR_EXT, GL_UNSIGNED_BYTE, Bitmap.bmBits);

	DeleteObject(hBitmap);

	return TRUE;
}