#include "mainwindow.h"

using namespace std;

MainWindow::MainWindow()
{
	m_hWnd		= NULL;
	m_Client	= NULL;
	m_hMenu		= NULL;
}

LRESULT WINAPI MainWindow::messageProc(HWND hWnd, UINT message, WPARAM wParam,
										   LPARAM lParam)
{
	MainWindow *exWindow = NULL;
	
	LRESULT returnCode = 0;

	if(!IsWindow(hWnd)) return FALSE;

	if(message == WM_NCCREATE)
	{
		LPCREATESTRUCT	pCreateStruct	= (LPCREATESTRUCT)lParam;

		SetWindowLong(hWnd, GWL_USERDATA,
			(LONG)pCreateStruct->lpCreateParams);
	}
	
	exWindow = (MainWindow *)GetWindowLong(hWnd, GWL_USERDATA);

	if(exWindow == NULL) return DefWindowProc(hWnd, message, wParam, lParam); 
	
	try
	{
		if(exWindow->handleMessage(hWnd, message, wParam, lParam,
			&returnCode)) return returnCode;

		//
	}
	catch(...)
	{
	}

	switch(message)
	{
	case WM_CLOSE:
		exWindow->CloseEvent();
		break;

	case WM_SIZE:
		exWindow->SizeEvent(LOWORD(lParam), HIWORD(lParam));
		break;

	case WM_PAINT:
		{
			PAINTSTRUCT PaintData;
			
			BeginPaint(hWnd, &PaintData);
			
			exWindow->PaintEvent(PaintData.hdc);

			EndPaint(hWnd, &PaintData);
		}
		break;

	case WM_KEYDOWN:
		exWindow->KeyDownEvent(wParam);
		break;

	case WM_LBUTTONDOWN:
	case WM_RBUTTONDOWN:
		exWindow->MouseDownEvent(wParam, LOWORD(lParam), HIWORD(lParam));
		break;

	case WM_MOUSEMOVE:
		exWindow->MouseMoveEvent(wParam, LOWORD(lParam), HIWORD(lParam));
		break;

	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	
	return 0;
}

BOOL MainWindow::handleMessage(HWND hWnd, UINT message, WPARAM wParam,
							  LPARAM lParam, LRESULT *returnCode)
{	
	return FALSE;
}

BOOL MainWindow::create(MainWindow *exWindow, char *title)
{
	BOOL bSucceded = create(exWindow, title, NULL, NULL);

	return bSucceded;
}

BOOL MainWindow::create(MainWindow *exWindow, char *title, HICON icon, HICON smallIcon)
{
	if(create(exWindow, title, WS_OVERLAPPEDWINDOW | WS_MAXIMIZE, 0, 0,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		NULL, icon, smallIcon,
		CreateSolidBrush(RGB(0, 0, 0))))
	{
		return TRUE;
	}
	
	return FALSE;
}

BOOL MainWindow::create(MainWindow *exWindow, char *title, DWORD windowStyle,
					   int x, int y, int width, int height, HMENU hMenu,
					   HICON icon, HICON smallIcon, HBRUSH hBrush)
{
	if(m_hWnd) return FALSE;
	
	WNDCLASSEX wc = {
		sizeof(WNDCLASSEX), CS_CLASSDC, messageProc, 0L, 0L,
		GetModuleHandle(NULL), icon, LoadCursor(NULL, IDC_ARROW),
		hBrush, NULL, title, smallIcon
	};

	RegisterClassEx(&wc);

	m_hWnd = CreateWindow(title, title,
		windowStyle, x, y, width, height,
		NULL, hMenu, wc.hInstance, exWindow ? (void *)exWindow : this);	

	if(m_hWnd) return TRUE;
	else return FALSE;
}

VOID MainWindow::show(BOOL maximized)
{
	ShowWindow(m_hWnd, maximized ? SW_MAXIMIZE : SW_SHOWDEFAULT);

	UpdateWindow(m_hWnd);
}

VOID MainWindow::destroy()
{
	DestroyWindow(m_hWnd);
}


BOOL MainWindow::flushMessage(bool* freeTime=0)
{		
	MSG message;
	
	if(!m_hWnd) return FALSE;

	ZeroMemory(&message, sizeof(message));
	
	if(PeekMessage(&message, NULL, 0, 0, PM_REMOVE))
	{
		TranslateMessage(&message);
		DispatchMessage(&message);

		if(freeTime) *freeTime = false;	// Posibily more messages in the queue.
	}
	else
	{
		if(freeTime) *freeTime = true; // No more messages.
	}

	if(message.message == WM_QUIT || message.message < 0) return FALSE;
	else return TRUE;
}

HWND MainWindow::handle()
{
	return m_hWnd;
}

HDC MainWindow::deviceHandle()
{
	return GetDC(handle());
}

HMENU MainWindow::menu()
{
	return m_hMenu;
}

VOID MainWindow::setOverlapped(BOOL overlapped)
{
	if(overlapped)
	{
		SetWindowLong(m_hWnd, GWL_STYLE, WS_OVERLAPPEDWINDOW);
	}
	else
	{
		SetWindowLong(m_hWnd, GWL_STYLE, WS_POPUP);
	}
}

BOOL MainWindow::say(char *what)
{
	return say(what, FALSE);
}

BOOL MainWindow::say(char *what, BOOL error)
{
	if(m_hWnd)
	{
		char *buffer = new char[32];

		GetWindowText(m_hWnd, buffer, 31);

		if(error) MessageBox(m_hWnd, what, buffer, MB_ICONEXCLAMATION);
		else MessageBox(m_hWnd, what, buffer, MB_OK);

		return TRUE;
	}

	return FALSE;
}

BOOL MainWindow::create(MainWindow *exWindow, char *title, BOOL maximized)
{
	BOOL bSucceded = create(exWindow, title, NULL, NULL);

	show(maximized);

	return bSucceded;
}

BOOL MainWindow::create(MainWindow *exWindow, char *title, LPCTSTR pMenu, HICON icon, HICON smallIcon, BOOL maximized, HBRUSH hBrush)
{
	BOOL bSucceded = FALSE;

	bSucceded = create(exWindow, title, pMenu, icon, smallIcon, hBrush);

	if(!bSucceded) return FALSE;

	return TRUE;
}

BOOL MainWindow::create(MainWindow *exWindow, char *title, LPCTSTR pMenu, HICON icon, HICON smallIcon, HBRUSH hBrush)
{
	m_hMenu = LoadMenu((HINSTANCE)GetModuleHandle(NULL), pMenu);

	if(!m_hMenu) return FALSE;

	if(create(exWindow, title, WS_OVERLAPPEDWINDOW | WS_MAXIMIZE, 0, 0,
		CW_USEDEFAULT,
		CW_USEDEFAULT,
		m_hMenu, icon, smallIcon,
		hBrush))
	{
		return TRUE;
	}

	return FALSE;
}

RECT MainWindow::clientRect()
{
	RECT clientRect = {0};
	
	GetClientRect(handle(), &clientRect);

	return clientRect;
}

int	MainWindow::width()
{
	return clientRect().right;
}

int	MainWindow::height()
{
	return clientRect().bottom;
}

void MainWindow::CloseEvent()
{
	destroy();
}

void MainWindow::SizeEvent(int width, int height)
{
}

void MainWindow::PaintEvent(HDC paintDC)
{
}

void MainWindow::MouseDownEvent(DWORD key, int x, int y)
{	
}

void MainWindow::MouseMoveEvent(DWORD key, int x, int y)
{
}

void MainWindow::KeyDownEvent(BYTE key)
{
}
