#include "openglwindow.h"

OpenGLWindow::OpenGLWindow() : MainWindow(), m_hRC(NULL)
{
}

BOOL OpenGLWindow::InitOpenGL(BYTE BitDepth)
{
	GLuint PixelFormat = NULL;
	
	static PIXELFORMATDESCRIPTOR PixelDec =
	{
		sizeof(PIXELFORMATDESCRIPTOR),				// Size Of This Pixel Format Descriptor
		1,											// Version Number
		PFD_DRAW_TO_WINDOW |						// Format Must Support Window
		PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
		PFD_DOUBLEBUFFER,							// Must Support Double Buffering
		PFD_TYPE_RGBA,								// Request An RGBA Format
		BitDepth,									// Select Our Color Depth
		0, 0, 0, 0, 0, 0,							// Color Bits Ignored
		0,											// No Alpha Buffer
		0,											// Shift Bit Ignored
		0,											// No Accumulation Buffer
		0, 0, 0, 0,									// Accumulation Bits Ignored
		16,											// 16Bit Z-Buffer (Depth Buffer)  
		0,											// No Stencil Buffer
		0,											// No Auxiliary Buffer
		PFD_MAIN_PLANE,								// Main Drawing Layer
		0,											// Reserved
		0, 0, 0										// Layer Masks Ignored
	};
	
	if(!deviceHandle()) return FALSE;

	if(!(PixelFormat = ChoosePixelFormat(deviceHandle(), &PixelDec))) return FALSE;
	
	if(!SetPixelFormat(deviceHandle(), PixelFormat, &PixelDec)) return FALSE;
	
	if(!(m_hRC = wglCreateContext(deviceHandle()))) return FALSE;
	
	if(!wglMakeCurrent(deviceHandle(), m_hRC)) return FALSE;

	// Init some basic states.

	if(!InitStates()) return FALSE;

	// Reset the viewport.

	ResetViewport(width(), height());

	cout << "GL Init OK." << endl;

	return TRUE;
}

void OpenGLWindow::ResetViewport(int Width, int Height)		// Resize the GL window.
{
	if(!Width || !Height) return;

	glViewport(0, 0, Width, Height);			// Reset the current viewport.
}

void OpenGLWindow::SwapBuffers()
{
	::SwapBuffers(deviceHandle());
}

BOOL OpenGLWindow::InitStates()
{
	// Set some basic states.

	glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);				// Black Background
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);								// The type of depth testing to do.
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Perspective Calculations

	return TRUE;
}

//

void OpenGLWindow::SizeEvent(int width, int height)
{
	ResetViewport(width, height);
}

void OpenGLWindow::PaintEvent(HDC paintDC)
{
	SwapBuffers();

	ValidateRect(handle(), NULL);
}