#include "skybox.h"
#define GL_CLAMP_TO_EDGE	0x812F						// Used for skybox textures
void SkyBox::Destroy()
{
	glDeleteTextures(6, m_SkyTex);

	GLObject::Destroy();

	cout << "Skybox destroyed." << endl;
}

void SkyBox::Load(Camera* pCamera)
{
	this->pCamera = pCamera;

	bool fail = false;
	
	if(!GLLoadBitmap("Data/ypos.bmp", m_SkyTex[0])) fail = true;
	if(!GLLoadBitmap("Data/xneg.bmp", m_SkyTex[1])) fail = true;
	if(!GLLoadBitmap("Data/yneg.bmp", m_SkyTex[2])) fail = true;
	if(!GLLoadBitmap("Data/xpos.bmp", m_SkyTex[3])) fail = true;
	if(!GLLoadBitmap("Data/zpos.bmp", m_SkyTex[4])) fail = true;
	if(!GLLoadBitmap("Data/zneg.bmp", m_SkyTex[5])) fail = true;

	if(fail)
		cout << "Skybox textures not found!" << endl;
	else
		cout << "Skybox textures loaded." << endl;
}

void SkyBox::Render()
{
	if(!Visible()) return;

	GLint mode = 0;
	glGetIntegerv(GL_RENDER_MODE, &mode);
	
	if(mode == GL_SELECT) return;	// Nothing to select in the sky box.
	
	save();

	// Reset and transform the view matrix.
	
	if(pCamera) // Do we have attached camera?
	{
		glLoadIdentity();

		Vector3 at = pCamera->Forward();
		
		gluLookAt(0, 0, 0, at.m_X, at.m_Y, at.m_Z, 0, 0, 1);
	}

	// We dont need depth testing and we definitely dont want lighting!

	glEnable(GL_TEXTURE_2D);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);

	// Just in case we set all vertices to white.
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	glRotatef(90.0f, 1.0f, 0.0f, 0.0f);	// Fix rotation.

	// Texture coordinates are slightly contracted to prevent texture repeat/clamping artifacts.
	// We dont have clamp-to-border in this version of OpenGL.

	
	// Render the front quad
	glBindTexture(GL_TEXTURE_2D, m_SkyTex[0]);
	glBegin(GL_QUADS);
	glTexCoord2f(0.001f, 0.001f); glVertex3f(  0.5f, -0.5f, -0.5f );
	glTexCoord2f(0.999f, 0.001f); glVertex3f( -0.5f, -0.5f, -0.5f );
	glTexCoord2f(0.999f, 0.999f); glVertex3f( -0.5f,  0.5f, -0.5f );
	glTexCoord2f(0.001f, 0.999f); glVertex3f(  0.5f,  0.5f, -0.5f );
	glEnd();

	// Render the left quad
	glBindTexture(GL_TEXTURE_2D, m_SkyTex[1]);
	glBegin(GL_QUADS);
	glTexCoord2f(0.001f, 0.001f); glVertex3f(  0.5f, -0.5f,  0.5f );
	glTexCoord2f(0.999f, 0.001f); glVertex3f(  0.5f, -0.5f, -0.5f );
	glTexCoord2f(0.999f, 0.999f); glVertex3f(  0.5f,  0.5f, -0.5f );
	glTexCoord2f(0.001f, 0.999f); glVertex3f(  0.5f,  0.5f,  0.5f );
	glEnd();

	// Render the back quad
	glBindTexture(GL_TEXTURE_2D, m_SkyTex[2]);
	glBegin(GL_QUADS);
	glTexCoord2f(0.001f, 0.001f); glVertex3f( -0.5f, -0.5f,  0.5f );
	glTexCoord2f(0.999f, 0.001f); glVertex3f(  0.5f, -0.5f,  0.5f );
	glTexCoord2f(0.999f, 0.999f); glVertex3f(  0.5f,  0.5f,  0.5f );
	glTexCoord2f(0.001f, 0.999f); glVertex3f( -0.5f,  0.5f,  0.5f );

	glEnd();

	// Render the right quad
	glBindTexture(GL_TEXTURE_2D, m_SkyTex[3]);
	glBegin(GL_QUADS);
	glTexCoord2f(0.001f, 0.001f); glVertex3f( -0.5f, -0.5f, -0.5f );
	glTexCoord2f(0.999f, 0.001f); glVertex3f( -0.5f, -0.5f,  0.5f );
	glTexCoord2f(0.999f, 0.999f); glVertex3f( -0.5f,  0.5f,  0.5f );
	glTexCoord2f(0.001f, 0.999f); glVertex3f( -0.5f,  0.5f, -0.5f );
	glEnd();

	// Render the top quad
	glBindTexture(GL_TEXTURE_2D, m_SkyTex[4]);
	glBegin(GL_QUADS);
	glTexCoord2f(0.001f, 0.999f); glVertex3f( -0.5f,  0.5f, -0.5f );
	glTexCoord2f(0.001f, 0.001f); glVertex3f( -0.5f,  0.5f,  0.5f );
	glTexCoord2f(0.999f, 0.001f); glVertex3f(  0.5f,  0.5f,  0.5f );
	glTexCoord2f(0.999f, 0.999f); glVertex3f(  0.5f,  0.5f, -0.5f );
	glEnd();

	// Render the bottom quad
	glBindTexture(GL_TEXTURE_2D, m_SkyTex[5]);
	glBegin(GL_QUADS);
	glTexCoord2f(0.001f, 0.001f); glVertex3f( -0.5f, -0.5f, -0.5f );
	glTexCoord2f(0.001f, 0.999f); glVertex3f( -0.5f, -0.5f,  0.5f );
	glTexCoord2f(0.999f, 0.999f); glVertex3f(  0.5f, -0.5f,  0.5f );
	glTexCoord2f(0.999f, 0.001f); glVertex3f(  0.5f, -0.5f, -0.5f );
	glEnd();

	GLObject::Render();

	restore();
}