#pragma once

#include "base.h"

/*

This is the base class for all objects in the world. It defines virtual methods which allows
for polymorphic rendering etc.

Transformation and Lighting can also be done using helper methods.

*/

class GLObject : public Vector3
{
protected:

	float sx;
	float sy;
	float sz;

	GLfloat diffuse[4];
	GLfloat specular[4];
	GLfloat shininess;

	bool m_Visible;

public:

	GLObject();

	GLObject(Vector3 v);

	virtual void Destroy();	// Release all data this object holds.

	virtual void light();

	virtual void transform();

	void save();		// Save state information before render.

	void restore();	// Restore state information.

	virtual void Render();

	void setShininess(float s);
	
	void setDiffuse(float r, float g, float b, float a=1.0f);

	void setSpecular(float r, float g, float b, float a=1.0f);
	
	void setScale(float x, float y, float z);

	void setScale(float s);

	bool& Visible();

	//

	static bool GLLoadBitmap(const char* filename, GLuint& texid); // Creates texture from a bitmap file.
};